@@ -29,6 +29,7 @@
 #include "qgis.h"
 #include "qgsunittypes.h"
 #include "qgsinterval.h"
+#include "qgsexpressionnode.h"
 
 class QgsFeature;
 class QgsGeometry;
@@ -41,34 +42,27 @@ class QgsDistanceArea;
 class QDomElement;
 class QgsExpressionContext;
 class QgsExpressionPrivate;
-class QgsExpressionNode;
 class QgsExpressionFunction;
 
 /**
  * \ingroup core
-Class for parsing and evaluation of expressions (formerly called "search strings").
-The expressions try to follow both syntax and semantics of SQL expressions.
+ * \brief Class for parsing and evaluation of expressions (formerly called "search strings").
+ * The expressions try to follow both syntax and semantics of SQL expressions.
 
 Usage:
-\code{.cpp}
-  QgsExpression exp("gid*2 > 10 and type not in ('D','F')");
-  if (exp.hasParserError())
-  {
-    // show error message with parserErrorString() and exit
-  }
-  QVariant result = exp.evaluate(feature, fields);
-  if (exp.hasEvalError())
-  {
-    // show error message with evalErrorString()
-  }
-  else
-  {
-    // examine the result
-  }
+\code{.py}
+  exp = QgsExpression("gid*2 > 10 and type not in ('D','F')")
+  if exp.hasParserError():
+      # show error message with parserErrorString() and exit
+
+  result = exp.evaluate(feature, fields)
+  if exp.hasEvalError():
+      # show error message with evalErrorString()
+  else:
+      # examine the result
 \endcode
 
-Three Value Logic
-=================
+\section value_logic Three Value Logic
 
 Similarly to SQL, this class supports three-value logic: true/false/unknown.
 Unknown value may be a result of operations with missing data (NULL). Please note
@@ -78,14 +72,12 @@ that NULL is different value than zero or an empty string. For example
 There is no special (three-value) 'boolean' type: true/false is represented as
 1/0 integer, unknown value is represented the same way as NULL values: NULL QVariant.
 
-Performance
-===========
+\section performance Performance
 
 For better performance with many evaluations you may first call prepare(fields) function
 to find out indices of columns and then repeatedly call evaluate(feature).
 
-Type conversion
-===============
+\section type_conversion Type conversion
 
 Operators and functions that expect arguments to be of a particular
 type automatically convert the arguments to that type, e.g. sin('2.1') will convert
@@ -95,8 +87,7 @@ If implicit or explicit conversion is invalid, the evaluation returns an error.
 Comparison operators do numeric comparison in case both operators are numeric (int/double)
 or they can be converted to numeric types.
 
-Implicit sharing
-================
+\section implicit_sharing Implicit sharing
 
 This class is implicitly shared, copying has a very low overhead.
 It is normally preferable to call `QgsExpression( otherExpression )` instead of
@@ -115,6 +106,54 @@ class CORE_EXPORT QgsExpression
     Q_DECLARE_TR_FUNCTIONS( QgsExpression )
   public:
 
+    /**
+     * Details about any parser errors that were found when parsing the expression.
+     * \since QGIS 3.0
+     */
+    struct CORE_EXPORT ParserError
+    {
+      enum ParserErrorType
+      {
+        Unknown = 0,  //!< Unknown error type.
+        FunctionUnknown = 1, //!< Function was unknown.
+        FunctionWrongArgs = 2, //!< Function was called with the wrong number of args.
+        FunctionInvalidParams = 3, //!< Function was called with invalid args.
+        FunctionNamedArgsError = 4 //!< Non named function arg used after named arg.
+      };
+
+      /**
+       * The type of parser error that was found.
+       */
+      ParserErrorType errorType = ParserErrorType::Unknown;
+
+      /**
+       * The message for the error at this location.
+       */
+      QString errorMsg;
+
+      /**
+       * The first line that contained the error in the parser.
+       * Depending on the error sometimes this doesn't mean anything.
+       */
+      int firstLine = 0;
+
+      /**
+       * The first column that contained the error in the parser.
+       * Depending on the error sometimes this doesn't mean anything.
+       */
+      int firstColumn = 0;
+
+      /**
+       * The last line that contained the error in the parser.
+       */
+      int lastLine = 0;
+
+      /**
+       * The last column that contained the error in the parser.
+       */
+      int lastColumn = 0;
+    };
+
     /**
      * Creates a new expression based on the provided string.
      * The string will immediately be parsed. For optimization
@@ -154,7 +193,7 @@ class CORE_EXPORT QgsExpression
     ~QgsExpression();
 
     /**
-     * Compares two expressions. The operator returns true
+     * Compares two expressions. The operator returns TRUE
      * if the expression string is equal.
      *
      * \since QGIS 3.0
@@ -169,49 +208,121 @@ class CORE_EXPORT QgsExpression
      */
     bool isValid() const;
 
-    //! Returns true if an error occurred when parsing the input expression
+    //! Returns TRUE if an error occurred when parsing the input expression
     bool hasParserError() const;
     //! Returns parser error
     QString parserErrorString() const;
 
-    //! Returns root node of the expression. Root node is null is parsing has failed
+    /**
+     * Returns parser error details including location of error.
+     * \since QGIS 3.0
+     */
+    QList<QgsExpression::ParserError> parserErrors() const;
+
+    /**
+     * Returns the root node of the expression.
+     *
+     * The root node is NULLPTR if parsing has failed.
+     */
     const QgsExpressionNode *rootNode() const;
 
     /**
-     * Get the expression ready for evaluation - find out column indexes.
+     * Gets the expression ready for evaluation - find out column indexes.
      * \param context context for preparing expression
      * \since QGIS 2.12
      */
     bool prepare( const QgsExpressionContext *context );
 
     /**
-     * Get list of columns referenced by the expression.
+     * Gets list of columns referenced by the expression.
      *
      * \note If the returned list contains the QgsFeatureRequest::AllAttributes constant then
      * all attributes from the layer are required for evaluation of the expression.
      * QgsFeatureRequest::setSubsetOfAttributes automatically handles this case.
      *
+     * \warning If the expression has been prepared via a call to QgsExpression::prepare(),
+     * or a call to QgsExpressionNode::prepare() for a node has been made, then parts of
+     * the expression may have been determined to evaluate to a static pre-calculatable value.
+     * In this case the results will omit attribute indices which are used by these
+     * pre-calculated nodes, regardless of their actual referenced columns.
+     * If you are seeking to use these functions to introspect an expression you must
+     * take care to do this with an unprepared expression.
+     *
      * \see referencedAttributeIndexes()
      */
     QSet<QString> referencedColumns() const;
 
     /**
-     * Return a list of all variables which are used in this expression.
+     * Returns a list of all variables which are used in this expression.
      * If the list contains a NULL QString, there is a variable name used
      * which is determined at runtime.
      *
+     * \note In contrast to the referencedColumns() function this method
+     * is not affected by any previous calls to QgsExpression::prepare(),
+     * or QgsExpressionNode::prepare().
+     *
      * \since QGIS 3.0
      */
     QSet<QString> referencedVariables() const;
 
     /**
-     * Return a list of field name indexes obtained from the provided fields.
+     * Returns a list of the names of all functions which are used in this expression.
+     *
+     * \note In contrast to the referencedColumns() function this method
+     * is not affected by any previous calls to QgsExpression::prepare(),
+     * or QgsExpressionNode::prepare().
+     *
+     * \since QGIS 3.2
+     */
+    QSet<QString> referencedFunctions() const;
+
+#ifndef SIP_RUN
+
+    /**
+     * Returns a list of all nodes which are used in this expression
+     *
+     * \note not available in Python bindings
+     * \since QGIS 3.2
+     */
+    QList<const QgsExpressionNode *> nodes( ) const;
+
+    /**
+     * Returns a list of all nodes of the given class which are used in this expression
+     *
+     * \note not available in Python bindings
+     * \since QGIS 3.2
+     */
+    template <class T>
+    QList<const T *> findNodes( ) const
+    {
+      QList<const T *> lst;
+      const QList<const QgsExpressionNode *> allNodes( nodes() );
+      for ( const auto &node : allNodes )
+      {
+        const T *n = dynamic_cast<const T *>( node );
+        if ( n )
+          lst << n;
+      }
+      return lst;
+    }
+#endif
+
+    /**
+     * Returns a list of field name indexes obtained from the provided fields.
+     *
+     * \warning If the expression has been prepared via a call to QgsExpression::prepare(),
+     * or a call to QgsExpressionNode::prepare() for a node has been made, then parts of
+     * the expression may have been determined to evaluate to a static pre-calculatable value.
+     * In this case the results will omit attribute indices which are used by these
+     * pre-calculated nodes, regardless of their actual referenced columns.
+     * If you are seeking to use these functions to introspect an expression you must
+     * take care to do this with an unprepared expression.
      *
      * \since QGIS 3.0
      */
     QSet<int> referencedAttributeIndexes( const QgsFields &fields ) const;
 
-    //! Returns true if the expression uses feature geometry for some computation
+    //! Returns TRUE if the expression uses feature geometry for some computation
     bool needsGeometry() const;
 
     // evaluation
@@ -231,11 +342,11 @@ class CORE_EXPORT QgsExpression
      */
     QVariant evaluate( const QgsExpressionContext *context );
 
-    //! Returns true if an error occurred when evaluating last input
+    //! Returns TRUE if an error occurred when evaluating last input
     bool hasEvalError() const;
     //! Returns evaluation error
     QString evalErrorString() const;
-    //! Set evaluation error (used internally by evaluation functions)
+    //! Sets evaluation error (used internally by evaluation functions)
     void setEvalErrorString( const QString &str );
 
     /**
@@ -249,7 +360,7 @@ class CORE_EXPORT QgsExpression
      * \param text string to test
      * \param context optional expression context
      * \param errorMessage will be filled with any error message from the validation
-     * \returns true if string is a valid expression
+     * \returns TRUE if string is a valid expression
      * \since QGIS 2.12
      */
     static bool checkExpression( const QString &text, const QgsExpressionContext *context, QString &errorMessage SIP_OUT );
@@ -262,14 +373,14 @@ class CORE_EXPORT QgsExpression
     void setExpression( const QString &expression );
 
     /**
-     * Return the original, unmodified expression string.
+     * Returns the original, unmodified expression string.
      * If there was none supplied because it was constructed by sole
      * API calls, dump() will be used to create one instead.
      */
     QString expression() const;
 
     /**
-     * Return an expression string, constructed from the internal
+     * Returns an expression string, constructed from the internal
      * abstract syntax tree. This does not contain any nice whitespace
      * formatting or comments. In general it is preferable to use
      * expression() instead.
@@ -277,7 +388,7 @@ class CORE_EXPORT QgsExpression
     QString dump() const;
 
     /**
-     * Return calculator used for distance and area calculations
+     * Returns calculator used for distance and area calculations
      * (used by $length, $area and $perimeter functions only)
      * \see setGeomCalculator()
      * \see distanceUnits()
@@ -287,10 +398,14 @@ class CORE_EXPORT QgsExpression
 
     /**
      * Sets the geometry calculator used for distance and area calculations in expressions.
-     * (used by $length, $area and $perimeter functions only). By default, no geometry
-     * calculator is set and all distance and area calculations are performed using simple
+     * (used by $length, $area and $perimeter functions only).
+     * If the geometry calculator is set to NULLPTR (default), prepare() will read variables
+     * from the expression context ("project_ellipsoid", "_project_transform_context" and
+     * "_layer_crs") to build a geometry calculator.
+     * If these variables does not exist and if setGeomCalculator() is not called,
+     * all distance and area calculations are performed using simple
      * Cartesian methods (ie no ellipsoidal calculations).
-     * \param calc geometry calculator. Ownership is not transferred. Set to a nullptr to force
+     * \param calc geometry calculator. Ownership is not transferred. Set to NULLPTR to force
      * Cartesian calculations.
      * \see geomCalculator()
      */
@@ -299,36 +414,40 @@ class CORE_EXPORT QgsExpression
     /**
      * Returns the desired distance units for calculations involving geomCalculator(), e.g., "$length" and "$perimeter".
      * \note distances are only converted when a geomCalculator() has been set
-     * \since QGIS 2.14
      * \see setDistanceUnits()
      * \see areaUnits()
+     * \since QGIS 2.14
      */
     QgsUnitTypes::DistanceUnit distanceUnits() const;
 
     /**
      * Sets the desired distance units for calculations involving geomCalculator(), e.g., "$length" and "$perimeter".
+     * If distance units are set to QgsUnitTypes::DistanceUnknownUnit (default), prepare() will read
+     * variables from the expression context ("project_distance_units") to determine distance units.
      * \note distances are only converted when a geomCalculator() has been set
-     * \since QGIS 2.14
      * \see distanceUnits()
      * \see setAreaUnits()
+     * \since QGIS 2.14
      */
     void setDistanceUnits( QgsUnitTypes::DistanceUnit unit );
 
     /**
      * Returns the desired areal units for calculations involving geomCalculator(), e.g., "$area".
      * \note areas are only converted when a geomCalculator() has been set
-     * \since QGIS 2.14
      * \see setAreaUnits()
      * \see distanceUnits()
+     * \since QGIS 2.14
      */
     QgsUnitTypes::AreaUnit areaUnits() const;
 
     /**
      * Sets the desired areal units for calculations involving geomCalculator(), e.g., "$area".
+     * If distance units are set to QgsUnitTypes::AreaUnknownUnit (default), prepare() will read
+     * variables from the expression context ("project_distance_units") to determine distance units.
      * \note areas are only converted when a geomCalculator() has been set
-     * \since QGIS 2.14
      * \see areaUnits()
      * \see setDistanceUnits()
+     * \since QGIS 2.14
      */
     void setAreaUnits( QgsUnitTypes::AreaUnit unit );
 
@@ -346,17 +465,26 @@ class CORE_EXPORT QgsExpression
     static QString replaceExpressionText( const QString &action, const QgsExpressionContext *context,
                                           const QgsDistanceArea *distanceArea = nullptr );
 
+    /**
+     * This function returns variables in each expression between [% and %].
+     *
+     * \param text The source string in which variables should be searched.
+     *
+     * \since QGIS 3.2
+     */
+    static QSet<QString> referencedVariables( const QString &text );
+
     /**
      * Attempts to evaluate a text string as an expression to a resultant double
      * value.
      * \param text text to evaluate as expression
      * \param fallbackValue value to return if text can not be evaluated as a double
      * \returns evaluated double value, or fallback value
-     * \since QGIS 2.7
      * \note this method is inefficient for bulk evaluation of expressions, it is intended
      * for one-off evaluations only.
+     * \since QGIS 2.7
      */
-    static double evaluateToDouble( const QString &text, const double fallbackValue );
+    static double evaluateToDouble( const QString &text, double fallbackValue );
 
     enum SpatialOperator
     {
@@ -371,19 +499,15 @@ class CORE_EXPORT QgsExpression
       soWithin,
     };
 
-    //! \note not available in Python bindings
-    static QList<QgsExpressionFunction *> sFunctions SIP_SKIP;
     static const QList<QgsExpressionFunction *> &Functions();
 
-    //! \note not available in Python bindings
-    static QStringList sBuiltinFunctions SIP_SKIP;
     static const QStringList &BuiltinFunctions();
 
     /**
      * Registers a function to the expression engine. This is required to allow expressions to utilize the function.
      * \param function function to register
-     * \param transferOwnership set to true to transfer ownership of function to expression engine
-     * \returns true on successful registration
+     * \param transferOwnership set to TRUE to transfer ownership of function to expression engine
+     * \returns TRUE on successful registration
      * \see unregisterFunction
      */
     static bool registerFunction( QgsExpressionFunction *function, bool transferOwnership = false );
@@ -395,12 +519,6 @@ class CORE_EXPORT QgsExpression
      */
     static bool unregisterFunction( const QString &name );
 
-    /**
-     * List of functions owned by the expression engine
-     * \note not available in Python bindings
-     */
-    static QList<QgsExpressionFunction *> sOwnedFunctions SIP_SKIP;
-
     /**
      * Deletes all registered functions whose ownership have been transferred to the expression engine.
      * \since QGIS 2.12
@@ -410,12 +528,12 @@ class CORE_EXPORT QgsExpression
     //! tells whether the identifier is a name of existing function
     static bool isFunctionName( const QString &name );
 
-    //! return index of the function in Functions array
+    //! Returns index of the function in Functions array
     static int functionIndex( const QString &name );
 
     /**
      * Returns the number of functions defined in the parser
-     *  \returns The number of function defined in the parser.
+     * \returns The number of function defined in the parser.
      */
     static int functionCount();
 
@@ -437,9 +555,9 @@ class CORE_EXPORT QgsExpression
      * Returns a string representation of a literal value, including appropriate
      * quotations where required.
      * \param value value to convert to a string representation
-     * \since QGIS 2.14
      * \see quotedString()
      * \see quotedColumnRef()
+     * \since QGIS 2.14
      */
     static QString quotedValue( const QVariant &value );
 
@@ -448,9 +566,9 @@ class CORE_EXPORT QgsExpression
      * quotations where required.
      * \param value value to convert to a string representation
      * \param type value type
-     * \since QGIS 2.14
      * \see quotedString()
      * \see quotedColumnRef()
+     * \since QGIS 2.14
      */
     static QString quotedValue( const QVariant &value, QVariant::Type type );
 
@@ -464,11 +582,16 @@ class CORE_EXPORT QgsExpression
      */
     static QString helpText( QString name );
 
+    /**
+     * Returns a string list of search tags for a specified function.
+     * \param name function name
+     * \since QGIS 3.12
+     */
+    static QStringList tags( const QString &name );
+
     /**
      * Returns the help text for a specified variable.
      * \param variableName name of variable
-     * \param showValue set to true to include current value of variable in help text
-     * \param value current value of variable to show in help text
      * \see helpText()
      * \since QGIS 2.12
      */
@@ -477,7 +600,7 @@ class CORE_EXPORT QgsExpression
     /**
      * Returns formatted help text for a variable.
      * \param description translated description of variable
-     * \param showValue set to true to include current value of variable in help text
+     * \param showValue set to TRUE to include current value of variable in help text
      * \param value current value of variable to show in help text
      * \see helpText()
      * \see variableHelpText()
@@ -495,10 +618,12 @@ class CORE_EXPORT QgsExpression
      * Formats an expression result for friendly display to the user. Truncates the result to a sensible
      * length, and presents text representations of non numeric/text types (e.g., geometries and features).
      * \param value expression result to format
-     * \returns formatted string, may contain HTML formatting characters
+     * \param htmlOutput set to TRUE to allow HTML formatting, or FALSE for plain text output
+     * \param maximumPreviewLength define the maximum character length of the preview
+     * \returns formatted string, may contain HTML formatting characters if \a htmlOutput is TRUE
      * \since QGIS 2.14
      */
-    static QString formatPreviewString( const QVariant &value );
+    static QString formatPreviewString( const QVariant &value, bool htmlOutput = true, int maximumPreviewLength = 60 );
 
     /**
      * Create an expression allowing to evaluate if a field is equal to a
@@ -510,82 +635,44 @@ class CORE_EXPORT QgsExpression
      */
     static QString createFieldEqualityExpression( const QString &fieldName, const QVariant &value );
 
+    /**
+     * Returns TRUE if the given \a expression is a simple "field=value" type expression.
+     *
+     * \param expression expression to test
+     * \param field will be set to the field name if the expression is a field equality expression
+     * \param value will be set to the value if the expression is a field equality expression
+     * \returns TRUE if the expression is a field equality expression
+     *
+     * \since QGIS 3.18
+     */
+    static bool isFieldEqualityExpression( const QString &expression, QString &field SIP_OUT, QVariant &value SIP_OUT );
+
+    /**
+     * Attempts to reduce a list of expressions to a single "field IN (val1, val2, ... )" type expression.
+     *
+     * This will only be possible if all the input expressions form simple "field=value" OR "field IN (value1, value2)" expressions, and all
+     * reference the same field name.
+     *
+     * Returns TRUE if the given \a expressions could be converted to an IN type expression.
+     *
+     * \param expressions expressions to test
+     * \param result will be set to the calculated "field IN (...)" expression, wherever possible
+     * \returns TRUE if the expression was converted to a field IN type expression
+     *
+     * \since QGIS 3.18
+     */
+    static bool attemptReduceToInClause( const QStringList &expressions, QString &result SIP_OUT );
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsExpression: '%1'>" ).arg( sipCpp->expression() );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
   private:
-    void initGeomCalculator();
-
-    struct HelpArg SIP_SKIP
-    {
-      HelpArg( const QString &arg, const QString &desc, bool descOnly = false, bool syntaxOnly = false,
-               bool optional = false, const QString &defaultVal = QString() )
-        : mArg( arg )
-        , mDescription( desc )
-        , mDescOnly( descOnly )
-        , mSyntaxOnly( syntaxOnly )
-        , mOptional( optional )
-        , mDefaultVal( defaultVal )
-      {}
-
-      QString mArg;
-      QString mDescription;
-      bool mDescOnly;
-      bool mSyntaxOnly;
-      bool mOptional;
-      QString mDefaultVal;
-    };
-
-    struct HelpExample SIP_SKIP
-    {
-      HelpExample( const QString &expression, const QString &returns, const QString &note = QString() )
-        : mExpression( expression )
-        , mReturns( returns )
-        , mNote( note )
-      {}
-
-      QString mExpression;
-      QString mReturns;
-      QString mNote;
-    };
-
-    struct HelpVariant SIP_SKIP
-    {
-      HelpVariant( const QString &name, const QString &description,
-                   const QList<QgsExpression::HelpArg> &arguments = QList<QgsExpression::HelpArg>(),
-                   bool variableLenArguments = false,
-                   const QList<QgsExpression::HelpExample> &examples = QList<QgsExpression::HelpExample>(),
-                   const QString &notes = QString() )
-        : mName( name )
-        , mDescription( description )
-        , mArguments( arguments )
-        , mVariableLenArguments( variableLenArguments )
-        , mExamples( examples )
-        , mNotes( notes )
-      {}
-
-      QString mName;
-      QString mDescription;
-      QList<QgsExpression::HelpArg> mArguments;
-      bool mVariableLenArguments;
-      QList<QgsExpression::HelpExample> mExamples;
-      QString mNotes;
-    };
-
-    struct Help SIP_SKIP
-    {
-      //! Constructor for expression help
-      Help() = default;
-
-      Help( const QString &name, const QString &type, const QString &description, const QList<QgsExpression::HelpVariant> &variants )
-        : mName( name )
-        , mType( type )
-        , mDescription( description )
-        , mVariants( variants )
-      {}
-
-      QString mName;
-      QString mType;
-      QString mDescription;
-      QList<QgsExpression::HelpVariant> mVariants;
-    };
+    void initGeomCalculator( const QgsExpressionContext *context );
 
     /**
      * Helper for implicit sharing. When called will create
@@ -597,10 +684,6 @@ class CORE_EXPORT QgsExpression
 
     QgsExpressionPrivate *d = nullptr;
 
-    static QHash<QString, Help> sFunctionHelpTexts;
-    static QHash<QString, QString> sVariableHelpTexts;
-    static QHash<QString, QString> sGroups;
-
     //! \note not available in Python bindings
     static void initFunctionHelp() SIP_SKIP;
     //! \note not available in Python bindings
