@@ -13,11 +13,11 @@ email                : marco.hugentobler at sourcepole dot com
  *                                                                         *
  ***************************************************************************/
 
-#ifndef QGSMULTIPOINTV2_H
-#define QGSMULTIPOINTV2_H
+#ifndef QGSMULTIPOINT_H
+#define QGSMULTIPOINT_H
 
 #include "qgis_core.h"
-#include "qgis.h"
+#include "qgis_sip.h"
 #include "qgsgeometrycollection.h"
 
 /**
@@ -29,7 +29,54 @@ email                : marco.hugentobler at sourcepole dot com
 class CORE_EXPORT QgsMultiPoint: public QgsGeometryCollection
 {
   public:
-    QgsMultiPoint();
+
+    /**
+     * Constructor for an empty multipoint geometry.
+     */
+    QgsMultiPoint() SIP_HOLDGIL;
+
+#ifndef SIP_RUN
+
+    /**
+     * Returns the point with the specified \a index.
+     *
+     * \since QGIS 3.16
+     */
+    QgsPoint *pointN( int index );
+#else
+
+    /**
+     * Returns the point with the specified \a index.
+     *
+     * An IndexError will be raised if no point with the specified index exists.
+     *
+     * \since QGIS 3.16
+     */
+    SIP_PYOBJECT pointN( int index ) SIP_TYPEHINT( QgsPoint );
+    % MethodCode
+    if ( a0 < 0 || a0 >= sipCpp->numGeometries() )
+    {
+      PyErr_SetString( PyExc_IndexError, QByteArray::number( a0 ) );
+      sipIsErr = 1;
+    }
+    else
+    {
+      return sipConvertFromType( sipCpp->pointN( a0 ), sipType_QgsPoint, NULL );
+    }
+    % End
+#endif
+
+#ifndef SIP_RUN
+
+    /**
+     * Returns the point with the specified \a index.
+     *
+     * \note Not available in Python bindings
+     *
+     * \since QGIS 3.16
+     */
+    const QgsPoint *pointN( int index ) const;
+#endif
 
     QString geometryType() const override;
     QgsMultiPoint *clone() const override SIP_FACTORY;
@@ -38,13 +85,14 @@ class CORE_EXPORT QgsMultiPoint: public QgsGeometryCollection
     void clear() override;
     QDomElement asGml2( QDomDocument &doc, int precision = 17, const QString &ns = "gml", QgsAbstractGeometry::AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const override;
     QDomElement asGml3( QDomDocument &doc, int precision = 17, const QString &ns = "gml", QgsAbstractGeometry::AxisOrder axisOrder = QgsAbstractGeometry::AxisOrder::XY ) const override;
-    QString asJson( int precision = 17 ) const override;
-    int nCoordinates() const override;
+    json asJsonObject( int precision = 17 ) const override SIP_SKIP;
+    int nCoordinates() const override SIP_HOLDGIL;
     bool addGeometry( QgsAbstractGeometry *g SIP_TRANSFER ) override;
     bool insertGeometry( QgsAbstractGeometry *g SIP_TRANSFER, int index ) override;
     QgsAbstractGeometry *boundary() const override SIP_FACTORY;
     int vertexNumberFromVertexId( QgsVertexId id ) const override;
     double segmentLength( QgsVertexId startVertex ) const override;
+    bool isValid( QString &error SIP_OUT, int flags = 0 ) const override SIP_HOLDGIL;
 
 #ifndef SIP_RUN
     void filterVertices( const std::function< bool( const QgsPoint & ) > &filter ) override;
@@ -56,7 +104,7 @@ class CORE_EXPORT QgsMultiPoint: public QgsGeometryCollection
      * \note Not available in Python. Objects will be automatically be converted to the appropriate target type.
      * \since QGIS 3.0
      */
-    inline const QgsMultiPoint *cast( const QgsAbstractGeometry *geom ) const
+    inline static const QgsMultiPoint *cast( const QgsAbstractGeometry *geom )
     {
       if ( geom && QgsWkbTypes::flatType( geom->wkbType() ) == QgsWkbTypes::MultiPoint )
         return static_cast<const QgsMultiPoint *>( geom );
@@ -85,4 +133,4 @@ class CORE_EXPORT QgsMultiPoint: public QgsGeometryCollection
 
 // clazy:excludeall=qstring-allocations
 
-#endif // QGSMULTIPOINTV2_H
+#endif // QGSMULTIPOINT_H
