@@ -27,7 +27,7 @@ class QgsProcessingProvider;
 /**
  * \class QgsProcessingFeedback
  * \ingroup core
- * Base class for providing feedback from a processing algorithm.
+ * \brief Base class for providing feedback from a processing algorithm.
  *
  * This base class implementation silently ignores all feedback reported by algorithms.
  * Subclasses of QgsProcessingFeedback can be used to log this feedback or report
@@ -63,10 +63,23 @@ class CORE_EXPORT QgsProcessingFeedback : public QgsFeedback
      */
     virtual void reportError( const QString &error, bool fatalError = false );
 
+    /**
+     * Pushes a warning informational message from the algorithm. This
+     * should only be used sparsely as to maintain the importance of visual
+     * queues associated to this type of message.
+     * \see pushInfo()
+     * \see pushCommandInfo()
+     * \see pushDebugInfo()
+     * \see pushConsoleInfo()
+     * \since QGIS 3.16.2
+     */
+    virtual void pushWarning( const QString &warning );
+
     /**
      * Pushes a general informational message from the algorithm. This can
      * be used to report feedback which is neither a status report or an
      * error, such as "Found 47 matching features".
+     * \see pushWarning()
      * \see pushCommandInfo()
      * \see pushDebugInfo()
      * \see pushConsoleInfo()
@@ -77,6 +90,7 @@ class CORE_EXPORT QgsProcessingFeedback : public QgsFeedback
      * Pushes an informational message containing a command from the algorithm.
      * This is usually used to report commands which are executed in an external
      * application or as subprocesses.
+     * \see pushWarning()
      * \see pushInfo()
      * \see pushDebugInfo()
      * \see pushConsoleInfo()
@@ -86,6 +100,7 @@ class CORE_EXPORT QgsProcessingFeedback : public QgsFeedback
     /**
      * Pushes an informational message containing debugging helpers from
      * the algorithm.
+     * \see pushWarning()
      * \see pushInfo()
      * \see pushCommandInfo()
      * \see pushConsoleInfo()
@@ -95,6 +110,7 @@ class CORE_EXPORT QgsProcessingFeedback : public QgsFeedback
     /**
      * Pushes a console feedback message from the algorithm. This is used to
      * report the output from executing an external command or subprocess.
+     * \see pushWarning()
      * \see pushInfo()
      * \see pushDebugInfo()
      * \see pushCommandInfo()
@@ -135,7 +151,7 @@ class CORE_EXPORT QgsProcessingFeedback : public QgsFeedback
  * \class QgsProcessingMultiStepFeedback
  * \ingroup core
  *
- * Processing feedback object for multi-step operations.
+ * \brief Processing feedback object for multi-step operations.
  *
  * A processing feedback object which proxies its calls to an underlying
  * feedback object, but scales overall progress reports to account
@@ -164,6 +180,7 @@ class CORE_EXPORT QgsProcessingMultiStepFeedback : public QgsProcessingFeedback
 
     void setProgressText( const QString &text ) override;
     void reportError( const QString &error, bool fatalError = false ) override;
+    void pushWarning( const QString &warning ) override;
     void pushInfo( const QString &info ) override;
     void pushCommandInfo( const QString &info ) override;
     void pushDebugInfo( const QString &info ) override;
