@@ -41,17 +41,22 @@ from qgis.PyQt.QtGui import QColor, QCursor
 from qgis.core import (QgsApplication, QgsCoordinateReferenceSystem,
                        QgsCoordinateTransform, QgsGeometry, QgsPointXY,
                        QgsProviderRegistry, QgsSettings, QgsProject)
-from qgis.gui import QgsRubberBand
+from qgis.gui import QgsRubberBand, QgsGui
 from qgis.utils import OverrideCursor
 
 with warnings.catch_warnings():
     warnings.filterwarnings("ignore", category=ResourceWarning)
     warnings.filterwarnings("ignore", category=ImportWarning)
-    from owslib.csw import CatalogueServiceWeb # spellok
+    from owslib.csw import CatalogueServiceWeb  # spellok
 
 from owslib.fes import BBox, PropertyIsLike
 from owslib.ows import ExceptionReport
 
+try:
+    from owslib.util import Authentication
+except ImportError:
+    pass
+
 from MetaSearch import link_types
 from MetaSearch.dialogs.manageconnectionsdialog import ManageConnectionsDialog
 from MetaSearch.dialogs.newconnectiondialog import NewConnectionDialog
@@ -66,7 +71,6 @@ BASE_CLASS = get_ui_class('maindialog.ui')
 
 
 class MetaSearchDialog(QDialog, BASE_CLASS):
-
     """main dialogue"""
 
     def __init__(self, iface):
@@ -94,9 +98,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.rubber_band.setWidth(5)
 
         # form inputs
-        self.startfrom = 0
+        self.startfrom = 1
         self.maxrecords = 10
         self.timeout = 10
+        self.disable_ssl_verification = False
         self.constraints = []
 
         # Servers tab
@@ -280,6 +285,9 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
             self.textMetadata.document().setDefaultStyleSheet(style)
             self.textMetadata.setHtml(metadata)
 
+            # clear results and disable buttons in Search tab
+            self.clear_results()
+
     def add_connection(self):
         """add new service"""
 
@@ -392,7 +400,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         if crsid != 4326:  # reproject to EPSG:4326
             src = QgsCoordinateReferenceSystem(crsid)
-            dest = QgsCoordinateReferenceSystem(4326)
+            dest = QgsCoordinateReferenceSystem("EPSG:4326")
             xform = QgsCoordinateTransform(src, dest, QgsProject.instance())
             minxy = xform.transform(QgsPointXY(extent.xMinimum(),
                                                extent.yMinimum()))
@@ -425,10 +433,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.constraints = []
 
         # clear all fields and disable buttons
-        self.lblResults.clear()
-        self.treeRecords.clear()
-
-        self.reset_buttons()
+        self.clear_results()
 
         # save some settings
         self.settings.setValue('/MetaSearch/returnRecords',
@@ -442,7 +447,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.catalog_password = self.settings.value('%s/password' % key)
 
         # start position and number of records to return
-        self.startfrom = 0
+        self.startfrom = 1
         self.maxrecords = self.spnRecords.value()
 
         # set timeout
@@ -461,8 +466,8 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         # even for a global bbox, if a spatial filter is applied, then
         # the CSW server will skip records without a bbox
         if bbox != ['-180', '-90', '180', '90']:
-            self.constraints.append(BBox(bbox,
-                                         crs='urn:ogc:def:crs:OGC:1.3:CRS84'))
+            self.constraints.append(BBox([miny, minx, maxy, maxx],
+                                         crs='urn:ogc:def:crs:EPSG::4326'))
 
         # keywords
         if self.leKeywords.text():
@@ -503,10 +508,10 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         self.treeRecords.clear()
 
-        position = self.catalog.results['returned'] + self.startfrom
+        position = self.catalog.results['returned'] + self.startfrom - 1
 
         msg = self.tr('Showing {0} - {1} of %n result(s)', 'number of results',
-                      self.catalog.results['matches']).format(self.startfrom + 1,
+                      self.catalog.results['matches']).format(self.startfrom,
                                                               position)
 
         self.lblResults.setText(msg)
@@ -536,12 +541,21 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.btnNext.setEnabled(disabled)
         self.btnLast.setEnabled(disabled)
 
+    def clear_results(self):
+        """clear search results"""
+
+        self.lblResults.clear()
+        self.treeRecords.clear()
+        self.reset_buttons()
+
     def record_clicked(self):
         """record clicked signal"""
 
         # disable only service buttons
         self.reset_buttons(True, False, False)
 
+        self.rubber_band.reset()
+
         if not self.treeRecords.selectedItems():
             return
 
@@ -562,7 +576,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         if record.bbox is not None:
             points = bbox_to_polygon(record.bbox)
             if points is not None:
-                src = QgsCoordinateReferenceSystem(4326)
+                src = QgsCoordinateReferenceSystem("EPSG:4326")
                 dst = self.map.mapSettings().destinationCrs()
                 geom = QgsGeometry.fromWkt(points)
                 if src.postgisSrid() != dst.postgisSrid():
@@ -638,24 +652,24 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         caller = self.sender().objectName()
 
         if caller == 'btnFirst':
-            self.startfrom = 0
+            self.startfrom = 1
         elif caller == 'btnLast':
-            self.startfrom = self.catalog.results['matches'] - self.maxrecords
+            self.startfrom = self.catalog.results['matches'] - self.maxrecords + 1
         elif caller == 'btnNext':
-            self.startfrom += self.maxrecords
-            if self.startfrom >= self.catalog.results["matches"]:
+            if self.startfrom > self.catalog.results["matches"] - self.maxrecords:
                 msg = self.tr('End of results. Go to start?')
                 res = QMessageBox.information(self, self.tr('Navigation'),
                                               msg,
                                               (QMessageBox.Ok |
                                                QMessageBox.Cancel))
                 if res == QMessageBox.Ok:
-                    self.startfrom = 0
+                    self.startfrom = 1
                 else:
                     return
+            else:
+                self.startfrom += self.maxrecords
         elif caller == "btnPrev":
-            self.startfrom -= self.maxrecords
-            if self.startfrom <= 0:
+            if self.startfrom == 1:
                 msg = self.tr('Start of results. Go to end?')
                 res = QMessageBox.information(self, self.tr('Navigation'),
                                               msg,
@@ -663,9 +677,13 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
                                                QMessageBox.Cancel))
                 if res == QMessageBox.Ok:
                     self.startfrom = (self.catalog.results['matches'] -
-                                      self.maxrecords)
+                                      self.maxrecords + 1)
                 else:
                     return
+            elif self.startfrom <= self.maxrecords:
+                self.startfrom = 1
+            else:
+                self.startfrom -= self.maxrecords
 
         try:
             with OverrideCursor(Qt.WaitCursor):
@@ -735,9 +753,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         if sname in keys:  # duplicate found
             msg = self.tr('Connection {0} exists. Overwrite?').format(sname)
             res = QMessageBox.warning(self, self.tr('Saving server'), msg,
-                                      QMessageBox.Yes | QMessageBox.No)
-            if res != QMessageBox.Yes:  # assign new name with serial
+                                      QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+            if res == QMessageBox.No:  # assign new name with serial
                 sname = serialize_string(sname)
+            elif res == QMessageBox.Cancel:
+                return
 
         # no dups detected or overwrite is allowed
         if caller in ['mActionAddAms', 'mActionAddAfs']:
@@ -748,8 +768,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.settings.endGroup()
 
         # open provider window
-        ows_provider = QgsProviderRegistry.instance().createSelectionWidget(stype[2],
-                                                                            self)
+        ows_provider = QgsGui.sourceSelectProviderRegistry().createSelectionWidget(
+            stype[2],
+            self,
+            Qt.Widget,
+            QgsProviderRegistry.WidgetMode.Embedded)
         service_type = stype[0]
 
         # connect dialog signals to iface slots
@@ -760,6 +783,7 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         elif service_type == 'OGC:WFS':
             def addVectorLayer(path, name):
                 self.iface.mainWindow().addVectorLayer(path, name, 'WFS')
+
             ows_provider.addVectorLayer.connect(addVectorLayer)
             conn_cmb = ows_provider.findChild(QWidget, 'cmbConnections')
             connect = 'connectToServer'
@@ -774,9 +798,11 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         elif service_type == 'ESRI:ArcGIS:FeatureServer':
             def addAfsLayer(path, name):
                 self.iface.mainWindow().addVectorLayer(path, name, 'afs')
+
             ows_provider.addVectorLayer.connect(addAfsLayer)
             conn_cmb = ows_provider.findChild(QComboBox)
             connect = 'connectToServer'
+
         ows_provider.setModal(False)
         ows_provider.show()
 
@@ -819,11 +845,21 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
 
         identifier = get_item_data(item, 'identifier')
 
+        self.disable_ssl_verification = self.disableSSLVerification.isChecked()
+        auth = None
+
+        if self.disable_ssl_verification:
+            try:
+                auth = Authentication(verify=False)
+            except NameError:
+                pass
+
         try:
             with OverrideCursor(Qt.WaitCursor):
-                cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout, # spellok
+                cat = CatalogueServiceWeb(self.catalog_url, timeout=self.timeout,  # spellok
                                           username=self.catalog_username,
-                                          password=self.catalog_password)
+                                          password=self.catalog_password,
+                                          auth=auth)
                 cat.getrecordbyid(
                     [self.catalog.records[identifier].identifier])
         except ExceptionReport as err:
@@ -896,15 +932,25 @@ class MetaSearchDialog(QDialog, BASE_CLASS):
         self.rubber_band.reset()
 
     def _get_csw(self):
-        """convenience function to init owslib.csw.CatalogueServiceWeb""" # spellok
+        """convenience function to init owslib.csw.CatalogueServiceWeb"""  # spellok
+
+        self.disable_ssl_verification = self.disableSSLVerification.isChecked()
+        auth = None
+
+        if self.disable_ssl_verification:
+            try:
+                auth = Authentication(verify=False)
+            except NameError:
+                pass
 
         # connect to the server
         with OverrideCursor(Qt.WaitCursor):
             try:
-                self.catalog = CatalogueServiceWeb(self.catalog_url, # spellok
+                self.catalog = CatalogueServiceWeb(self.catalog_url,  # spellok
                                                    timeout=self.timeout,
                                                    username=self.catalog_username,
-                                                   password=self.catalog_password)
+                                                   password=self.catalog_password,
+                                                   auth=auth)
                 return True
             except ExceptionReport as err:
                 msg = self.tr('Error connecting to service: {0}').format(err)
